// Stubs, wrappers and test of cp_{fb,vt} from spu
//
// Jonathan Adamczewski <jonathan@brnz.org>


#include "cp_vt.h"
#include "cp_fb.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <unistd.h>

#include <spu_mfcio.h>


// Necessary setup for __linux_syscall() - based on those in newlib

#define __NR_ioctl 54

/* System callbacks from the SPU. See kernel source file
 *    arch/powerpc/include/asm/spu.h.  */
struct spu_syscall_block
{
	unsigned long long nr_ret;    /* System call nr and return value.  */
	unsigned long long parm[6];   /* System call arguments.  */
};

int __linux_syscall (struct spu_syscall_block *s);

int ioctl(unsigned int d, unsigned int request, unsigned long a) {
	struct spu_syscall_block s = {
		__NR_ioctl,
		{ d, request, a, 0, 0, 0 }
	};
	return __linux_syscall(&s);
}


// Wrappers to handle the nasty include mess we have going on...
void* mmap(void *start, size_t length, int prot, int flags,
		   int fd, off_t offset) {
	unsigned long long r = mmap_eaddr((unsigned long long)(uint32_t)start, length, prot, flags, fd, offset);
	return (void*)(long)r;
}

int munmap(void* start, size_t length) {
	return munmap_eaddr((unsigned long long)(uint32_t)start, length);
}

int __nldbl_snprintf(char *str, size_t size, const char *format, ...) {
	va_list arglist;
	va_start(arglist, format);
	int r = vsnprintf(str,size,format,arglist);
	va_end(arglist);
	return r;
}

int __nldbl_printf(const char *format, ...) {
	va_list arglist;
	va_start(arglist, format);
	int r = vprintf(format,arglist);
	va_end(arglist);
	return r;
}


int main()
{
	cp_vt vt;
	cp_fb fb;

	// Allocate some scratch space for syscalls
	uint32_t size = getpagesize();
	unsigned long long mmap_res = mmap_eaddr(0ULL, size, PROT_READ|PROT_WRITE, MAP_ANONYMOUS|MAP_PRIVATE, -1, 0);
	if(mmap_res == MAP_FAILED_EADDR) {
		perror("Could not mmap space for syscall use.");
		exit(EXIT_FAILURE);
	}

	void* cp_vt_syscall_space = (void*)(uint32_t)mmap_res;
	void* cp_fb_syscall_space = (void*)(uint32_t)(mmap_res + 128);

	// Set up the vt and fb
	cp_vt_open_graphics(&vt, cp_vt_syscall_space);
	cp_fb_open(&fb, cp_fb_syscall_space);

	// Draw something...
	int frame_ndx = 0;
	int p = fb.stride * fb.h * 4;
	for(int f = 0; f < 900; ++f) {
		for(int o = 0; o < p; o += 16384) {
			mfc_put((void*)(f*16+(o>>10)), fb.draw_addr[frame_ndx]+o, 16384, 0, 0, 0);
		}
		cp_fb_wait_vsync(&fb);
		cp_fb_flip(&fb, frame_ndx);
		frame_ndx ^= 1;
	}

	cp_vt_close(&vt);
	cp_fb_close(&fb);

	return EXIT_SUCCESS;
}

